/*
 * WScout - Lightweight PCAP visualizer.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#ifndef WSCOUT_GUI_HEX_DUMP_DOCK_HXX_
# define WSCOUT_GUI_HEX_DUMP_DOCK_HXX_

# include <sstream>
# include <iomanip>
# include <locale>

# include "wscout_gui_hex_dump_dock.hh"

namespace wscout
{

  namespace gui
  {

    inline
    QString
    HexDumpDock::filterBlanksOut(const QString& str)
    {
      QString	r;

      for (int i = 0; i < str.size(); ++i)
	if (not str[i].isSpace())
	  r.push_back(str[i]);
      return r;
    }

    inline
    void
    HexDumpDock::setup(const void* data, size_t len)
    {
      const unsigned char* const  b = static_cast<const unsigned char*> (data);
      const unsigned char* const  e = b + len;

      std::ostringstream	addresses;
      std::ostringstream	hex;
      std::ostringstream	ascii;

      addresses << std::hex << std::uppercase;
      hex << std::hex << std::uppercase;
      for (const unsigned char* c = b; c < e; c += lwidth)
	{
	  if (c != b)
	    {
	      hex << '\n';
	      ascii << '\n';
	      addresses << '\n';
	    }
	  addresses << std::setfill ('0')
		    << std::setw (addr_len * 2)
		    << (c - b);
	  for (unsigned i = 0; i < lwidth and c + i < e; i += bwidth)
	    {
	      if (i)
		hex << bigsep;
	      for (unsigned j = 0; j < bwidth and c + i + j < e; ++j)
		{
		  if (j)
		    hex << smallsep;
		  hex << std::setfill ('0') << std::setw (2)
		      << unsigned (c[i + j]);
		  ascii << (std::isprint(c[i + j]) ? char (c[i + j]) : '.');
		}
	    }
	}

      addresses_.setPlainText(addresses.str().c_str());
      hexData_.setPlainText(hex.str().c_str());
      asciiData_.setPlainText(ascii.str().c_str());
    }

  } // End of namespace gui.

} // End of namespace wscout.

#endif // ! WSCOUT_GUI_HEX_DUMP_DOCK_HXX_
