/*
 * WScout - Lightweight PCAP visualizer.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#include <wipal/phy/prism_header.hh>

#include "wscout_gui_prism_header.hh"

namespace wscout
{

  namespace gui
  {

    PrismHeader::next_header
    PrismHeader::setup(const void*		h,
		       size_t			caplen,
		       const Trace&		trace,
		       const frame_descriptor&	fd)
    {
      const prism::header* const prism = static_cast<const prism::header*> (h);
      const bool		 swapped = (fd.swapped() xor
					    trace.swapContent());

      if (caplen < prism->len(caplen, swapped))
	{
	  setTruncated();
	  return next_header (protocol::NONE, 0);
	}

      setUpdatesEnabled(false);
      setTruncated(false);
      setTime(prism->hosttime.get(swapped), prism->mactime.get(swapped));
      setRate(prism->rate.get(swapped) / 2.);
      setChannel(prism->channel.get(swapped));
      setSignal(prism->rssi.get(swapped),
		prism->signal.get(swapped),
		prism->noise.get(swapped));
      setUpdatesEnabled(true);

      return next_header (protocol::IEEE802_11,
			  prism->decapsulate(caplen, swapped));
    }

  } // End of namespace gui.

} // End of namespace wscout.
