/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_MICROSECONDS_STAMP_HXX_
# define TOOL_MICROSECONDS_STAMP_HXX_

# include "microseconds_stamp.hh"

namespace tool
{

  template <class T>
  microseconds_stamp<T>::
  microseconds_stamp(const tool::microseconds&	microseconds,
		     const T&			super):
    T (super),
    microseconds_ (microseconds)
  {
  }

  template <class T>
  template <class U>
  microseconds_stamp<T>::microseconds_stamp(const microseconds_stamp<U>& msst):
    T (msst),
    microseconds_ (msst.microseconds())
  {
  }

  template <class T>
  const tool::microseconds&
  microseconds_stamp<T>::microseconds() const
  {
    return microseconds_;
  }

  template <class T>
  tool::microseconds&
  microseconds_stamp<T>::microseconds()
  {
    return microseconds_;
  }

  template <class T>
  bool
  microseconds_stamp<T>::operator == (const microseconds_stamp& rhs) const
  {
    return T::operator == (rhs);
  }

  template <class T>
  bool
  microseconds_stamp<T>::operator != (const microseconds_stamp& rhs) const
  {
    return T::operator != (rhs);
  }

} // End of namespace tool.

WP_HASH_NAMESPACE_BEGIN

  template <class T>
  size_t
  hash< tool::microseconds_stamp<T> >::
  operator () (const tool::microseconds_stamp<T>& s) const
  {
    return hash<T>::operator () (s);
  }

WP_HASH_NAMESPACE_END

#endif // TOOL_MICROSECONDS_STAMP_HXX_
