/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_STATS_ACTIVITY_HH_
# define WIFI_FRAME_STATS_ACTIVITY_HH_

# include <wipal/wifi/frame/stats/abstract_stats.hh>
# include <wipal/wifi/frame/stats/epoch_tracker.hh>

namespace wifi
{

  namespace frame
  {

    namespace stats
    {

      struct activity: public abstract_stats
      {
	void		account_frame(const tool::microseconds&	timestamp,
				      unsigned			len,
				      unsigned			type,
				      unsigned			subtype,
				      bool			from_ap);

	void		restart();
	std::ostream&	print(std::ostream& o) const;

      private:
	enum { epoch_length = 60 * 1000000 }; // 1 minute.

	struct epoch
	{
	  epoch();

	  unsigned	count;
	  unsigned	ap_count;
	  unsigned	type_count[4];
	  unsigned	subtype_count[4][16];
	  unsigned	size;
	  unsigned	ap_size;
	  unsigned	type_size[4];
	  unsigned	subtype_size[4][16];
	};

      private:
	epoch_tracker<epoch, epoch_length>	tracker_;
      };

    } // End of namespace wifi::frame::stats.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

# include "activity.hxx"

#endif // ! WIFI_FRAME_STATS_ACTIVITY_HH_
