/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_STATS_ACTIVITY_HXX_
# define WIFI_FRAME_STATS_ACTIVITY_HXX_

# include <cstring>
# include <cassert>

# include <boost/foreach.hpp>

# include <wipal/wifi/frame/stats/activity.hh>

namespace wifi
{

  namespace frame
  {

    namespace stats
    {

      inline
      void
      activity::account_frame(const tool::microseconds&	timestamp,
			      unsigned			len,
			      unsigned			type,
			      unsigned			subtype,
			      bool			from_ap)
      {
	tracker_.update(timestamp);

	epoch&	e = tracker_.current();

	++e.count;
	++e.type_count[type];
	++e.subtype_count[type][subtype];
	e.size += len;
	e.type_size[type] += len;
	e.subtype_size[type][subtype] += len;
	if (from_ap)
	  {
	    ++e.ap_count;
	    e.ap_size += len;
	  }
      }

      inline
      void
      activity::restart()
      {
	tracker_.restart();
      }

      inline
      std::ostream&
      activity::print(std::ostream& o) const
      {
	BOOST_FOREACH(const epoch& e, tracker_.epochs())
	  o << "  "
	    << e.count << ' '
	    << e.size << ' '
	    << e.type_size[type::management] << ' '
	    << e.type_size[type::data] << ' '
	    << e.ap_size << '\n';
	return o << std::flush;
      }

      inline
      activity::epoch::epoch(): count (0), ap_count (0), size (0), ap_size (0)
      {
	memset(type_count, 0, sizeof (type_count));
	memset(type_size, 0, sizeof (type_size));
	memset(subtype_count, 0, sizeof (subtype_count));
	memset(subtype_size, 0, sizeof (subtype_size));
      }

    } // End of namespace wifi::frame::stats.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

#endif // ! WIFI_FRAME_STATS_ACTIVITY_HXX_
